package cn.lttc.dpworkshopmanageserver.biz.filing.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.lttc.dpworkshopmanageserver.biz.filing.dto.PlanOrderDto;
import cn.lttc.dpworkshopmanageserver.biz.filing.entity.Filing;
import cn.lttc.dpworkshopmanageserver.biz.filing.mapper.FilingMapper;
import cn.lttc.dpworkshopmanageserver.biz.filing.param.FilingParam;
import cn.lttc.dpworkshopmanageserver.biz.filing.service.IFilingService;
import cn.lttc.dpworkshopmanageserver.biz.receive.eneity.Receive;
import cn.lttc.dpworkshopmanageserver.biz.receive.mapper.ReceiveMapper;
import cn.lttc.dpworkshopmanageserver.biz.stock.param.StockParam;
import cn.lttc.dpworkshopmanageserver.core.context.login.LoginContextHolder;
import cn.lttc.dpworkshopmanageserver.core.exception.ServiceException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author soul
 * @version 1.0.0
 * @title FilingServiceImpl
 * @description <TODO description class purpose>
 * @create 2022/11/16 14:02
 **/
@Service
public class FilingServiceImpl extends ServiceImpl<FilingMapper, Filing> implements IFilingService {
    @Resource
    private FilingMapper filingMapper;
    @Resource
    private ReceiveMapper reveiveMappper;

    @Override
    public void add(FilingParam filingParam) {
        Filing filing = new Filing();
        BeanUtil.copyProperties(filingParam, filing);
        filing.setOrderStatus(0);
        filing.setFilingDate(DateUtil.date(new Date()));
        filing.setFilingUser(LoginContextHolder.me().getSysLoginUser().getId());
        this.save(filing);
    }

    @Override
    public void edit(FilingParam filingParam) {
        Filing filing = this.getById(filingParam.getId());
        BeanUtil.copyProperties(filingParam, filing);
        this.updateById(filing);
    }

    @Override
    public void delete(List<FilingParam> filingParamLst) {
        if (CollectionUtil.isEmpty(filingParamLst)) {
            return;
        } else {
            filingParamLst.forEach(item -> {
                List<Receive> receives = reveiveMappper.selectList(new QueryWrapper<Receive>().lambda().eq(Receive::getOrderId, item.getId()));
                if (CollectionUtil.isNotEmpty(receives)) {
                    throw new ServiceException(500, "该订单已领活，不允许删除！");
                }
                Filing filing = this.getById(item.getId());
                filing.setOrderStatus(3);
                this.updateById(filing);
            });
        }
    }
    @Override
    public void archive(StockParam stockParam) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("order_no", stockParam.getOrderNo());
        List<Filing> filingList = filingMapper.selectList(qw);
        filingList.forEach(item->{
                Filing filing = this.getById(item.getId());
                filing.setStatus(true);
                this.updateById(filing);
        });
//        if (CollectionUtil.isEmpty(filingParamLst)) {
//            return;
//        } else {
//            filingParamLst.forEach(item -> {
//                System.out.println("========="+item.getOrderNo());
////                List<Receive> receives = reveiveMappper.selectList(new QueryWrapper<Receive>().lambda().eq(Receive::getOrderId, item.getId()));
////                if (CollectionUtil.isNotEmpty(receives)) {
////                    throw new ServiceException(500, "该订单已领活，不允许删除！");
////                }
////                Filing filing = this.getById(item.getId());
////                filing.setOrderStatus(3);
////                this.updateById(filing);
//            });
//        }
    }

    @Override
    public List<PlanOrderDto>   getPlanOrderData() {
        List<Filing> filings = filingMapper.selectList(new QueryWrapper<Filing>().lambda().eq(Filing::getStatus, 1).select(Filing::getOrderNo, Filing::getStockQty));
        Map<String, Integer> collects = filings.stream().collect(Collectors.groupingBy(Filing::getOrderNo, Collectors.summingInt(Filing::getStockQty)));
        List<PlanOrderDto> planOrderDtos = new ArrayList<>();
        collects.forEach((orderNo, qty) -> {
            PlanOrderDto planOrderDto = new PlanOrderDto();
            planOrderDto.setOrderNo(orderNo);
            planOrderDto.setQty(qty);
            planOrderDtos.add(planOrderDto);
        });
        return planOrderDtos;
    }

    @Override
    public List<Filing> getOrderList() {
        return filingMapper.selectList(new QueryWrapper<Filing>().lambda().ne(Filing::getOrderStatus, 3));
    }

    @Override
    public List<Filing> listAvailable() {
        return filingMapper.selectList(new QueryWrapper<Filing>().lambda().ne(Filing::getOrderStatus, 3).eq(Filing::getStatus, false));
    }

    @Override
    public List<Filing> getFilterData() {
        DateTime date = DateUtil.offset(DateUtil.date(new Date()), DateField.DAY_OF_MONTH, -2);
        return filingMapper.getFilterData(DateUtil.formatDate(date));
    }
    @Override
    public List<Filing> getFilingList(Integer status) {
        List<Filing> filings = filingMapper.selFilingInfoMapper(status);
        return filings;
    }
}
