package cn.lttc.dpworkshopmanageserver.biz.team.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.lttc.dpworkshopmanageserver.biz.filing.entity.Filing;
import cn.lttc.dpworkshopmanageserver.biz.filing.mapper.FilingMapper;
import cn.lttc.dpworkshopmanageserver.biz.team.dto.TeamDto;
import cn.lttc.dpworkshopmanageserver.biz.team.entity.Team;
import cn.lttc.dpworkshopmanageserver.biz.team.mapper.TeamMapper;
import cn.lttc.dpworkshopmanageserver.biz.team.param.TeamParam;
import cn.lttc.dpworkshopmanageserver.biz.team.service.ITeamService;
import cn.lttc.dpworkshopmanageserver.biz.workshop.entity.Workshop;
import cn.lttc.dpworkshopmanageserver.biz.workshop.mapper.WorkshopMapper;
import cn.lttc.dpworkshopmanageserver.biz.workshop.param.WorkshopParam;
import cn.lttc.dpworkshopmanageserver.biz.workshop.service.IWorkshopService;
import cn.lttc.dpworkshopmanageserver.core.context.login.LoginContextHolder;
import cn.lttc.dpworkshopmanageserver.core.exception.ServiceException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author soul
 * @version 1.0.0
 * @title FilingServiceImpl
 * @description <TODO description class purpose>
 * @create 2022/11/16 14:02
 **/
@Service
public class TeamServiceImpl extends ServiceImpl<TeamMapper, Team> implements ITeamService {
    @Resource
    private TeamMapper teamMapper;
    @Resource
    private FilingMapper filingMapper;

    @Override
    public void add(TeamParam teamParam) {
        Team team = new Team();
        BeanUtil.copyProperties(teamParam, team);
//        workshop.setStatus(1);
        team.setCreatedDate(DateUtil.date(new Date()));
        team.setCreatedUser(LoginContextHolder.me().getSysLoginUser().getId());
        this.save(team);
    }

    @Override
    public void edit(TeamParam teamParam) {
        Team team = this.getById(teamParam.getId());
        Workshop workshop = teamParam.getWorkshop();
        BeanUtil.copyProperties(teamParam, team);
        team.setUpdatedDate(DateUtil.date(new Date()));
        team.setUpdatedUser(LoginContextHolder.me().getSysLoginUser().getId());
        this.updateById(team);
    }
//
    @Override
    public void delete(List<TeamParam> teamParamList) {
        if (CollectionUtil.isEmpty(teamParamList)) {
            return;
        } else {
            teamParamList.forEach(item -> {
                List<Filing> filings = filingMapper.selectList(new QueryWrapper<Filing>().lambda().eq(Filing::getTeamId,
                        item.getId()).ne(Filing::getOrderStatus,3));
                if (CollectionUtil.isNotEmpty(filings)) {
                    throw new ServiceException(500, "该班组已参与建档，不允许删除！");
                }
                this.removeById(item.getId());
            });
        }
    }
//
//    @Override
//    public List<PlanOrderDto> getPlanOrderData() {
//        List<Filing> filings = filingMapper.selectList(new QueryWrapper<Filing>().lambda().eq(Filing::getStatus, 1).select(Filing::getOrderNo, Filing::getStockQty));
//        Map<String, Integer> collects = filings.stream().collect(Collectors.groupingBy(Filing::getOrderNo, Collectors.summingInt(Filing::getStockQty)));
//        List<PlanOrderDto> planOrderDtos = new ArrayList<>();
//        collects.forEach((orderNo, qty) -> {
//            PlanOrderDto planOrderDto = new PlanOrderDto();
//            planOrderDto.setOrderNo(orderNo);
//            planOrderDto.setQty(qty);
//            planOrderDtos.add(planOrderDto);
//        });
//        return planOrderDtos;
//    }
//
//    @Override
//    public List<TeamDto> getTeamList() {
//        List<TeamDto> teams = teamMapper.selTeamInfoMapper();
//        return teams;
//    }
    @Override
    public List<Team> getTeamList() {
        List<Team> teams = teamMapper.selTeamInfoMapper();
        return teams;
    }
    @Override
    public List<Team> getTeamListByWorkshopId(String workshopid) {
        QueryWrapper<Team> qw = new QueryWrapper<>();
        qw.eq("workshop_id",Long.parseLong(workshopid));
        List<Team> teams = teamMapper.selectList(qw);
        return teams;
    }

//
//    @Override
//    public List<Filing> listAvailable() {
//        return workshopMapper.selectList(new QueryWrapper<Filing>().lambda().ne(Filing::getOrderStatus, 3).eq(Filing::getStatus, true));
//    }
//
//    @Override
//    public List<Filing> getFilterData() {
//        DateTime date = DateUtil.offset(DateUtil.date(new Date()), DateField.DAY_OF_MONTH, -2);
//        return workshopMapper.getFilterData(DateUtil.formatDate(date));
//    }
}
