/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package cn.lttc.dpworkshopmanageserver.core.auth.context;

import cn.hutool.core.util.ObjectUtil;
import cn.lttc.dpworkshopmanageserver.core.auth.service.IAuthService;
import cn.lttc.dpworkshopmanageserver.core.context.login.LoginContext;
import cn.lttc.dpworkshopmanageserver.core.exception.AuthException;
import cn.lttc.dpworkshopmanageserver.core.exception.enums.AuthExceptionEnum;
import cn.lttc.dpworkshopmanageserver.core.login.pojo.SysLoginUser;
import cn.lttc.dpworkshopmanageserver.core.user.entity.SysUser;
import cn.lttc.dpworkshopmanageserver.core.user.service.ISysUserService;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 登录用户上下文实现类
 *
 * @author xuyuxiang
 * @date 2020/3/13 12:19
 */
@Component
public class LoginContextSpringSecurityImpl implements LoginContext {

    @Resource
    private IAuthService authService;

    @Resource
    private ISysUserService sysUserService;

    private LoginContextSpringSecurityImpl() {

    }

    /**
     * 获取当前登录用户
     *
     * @author xuyuxiang
     * @date 2020/3/13 14:42
     */
    @Override
    public SysLoginUser getSysLoginUser() {
        Authentication authentication = authService.getAuthentication();
        if (ObjectUtil.isEmpty(authentication) || authentication.getPrincipal() instanceof String) {
            throw new AuthException(AuthExceptionEnum.NO_LOGIN_USER);
        } else {
            return (SysLoginUser) authentication.getPrincipal();
        }
    }

    @Override
    public SysLoginUser getSysLoginUserUpToDate() {
        SysLoginUser sysLoginUser = this.getSysLoginUser();
        Long loginUserId = sysLoginUser.getId();
        SysUser sysUser = sysUserService.getById(loginUserId);
        //构造SysLoginUser
        return authService.genSysLoginUser(sysUser);
    }
}
