package cn.lttc.dpworkshopmanageserver.core.user.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.crypto.digest.BCrypt;
import cn.lttc.dpworkshopmanageserver.core.context.login.LoginContextHolder;
import cn.lttc.dpworkshopmanageserver.core.exception.AuthException;
import cn.lttc.dpworkshopmanageserver.core.exception.ServiceException;
import cn.lttc.dpworkshopmanageserver.core.exception.enums.AuthExceptionEnum;
import cn.lttc.dpworkshopmanageserver.core.user.dto.SysUserDto;
import cn.lttc.dpworkshopmanageserver.core.user.entity.SysUser;
import cn.lttc.dpworkshopmanageserver.core.user.mapper.SysUserMapper;
import cn.lttc.dpworkshopmanageserver.core.user.param.ModifyPasswordParam;
import cn.lttc.dpworkshopmanageserver.core.user.param.SysUserParam;
import cn.lttc.dpworkshopmanageserver.core.user.service.ISysUserService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author soul
 * @version 1.0.0
 * @title SysUserImpl
 * @description <TODO description class purpose>
 * @create 2022/11/16 14:48
 **/
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements ISysUserService {

    @Resource
    private SysUserMapper sysUserMapper;

    @Override
    public void add(SysUserParam sysUserParam) {
        if(!this.checkUserAccount(sysUserParam.getAccount())){
            throw new ServiceException(500,"用户名已存在！");
        }
        SysUser sysUser = new SysUser();
        BeanUtil.copyProperties(sysUserParam, sysUser);
        if (ObjectUtil.isNotEmpty(sysUserParam.getPassword())) {
            sysUser.setPassword(BCrypt.hashpw(sysUser.getPassword(), BCrypt.gensalt()));
        }
        sysUser.setStatus(1);
        sysUser.setCreateUser(LoginContextHolder.me().getSysLoginUser().getId());
        sysUser.setCreateDate(DateUtil.date(new Date()));
        this.save(sysUser);
    }

    @Override
    public void edit(SysUserParam sysUserParam) {
        Long sysUserId = sysUserParam.getId();
        SysUser sysUser = this.getById(sysUserId);
        BeanUtil.copyProperties(sysUserParam, sysUser);
        if (ObjectUtil.isNotEmpty(sysUserParam.getPassword())) {
            sysUser.setPassword(BCrypt.hashpw(sysUser.getPassword(), BCrypt.gensalt()));
        }
        this.updateById(sysUser);
    }

    @Override
    public List<SysUserDto> listUser() {
        List<SysUser> list = sysUserMapper.selectList(new QueryWrapper<SysUser>().lambda().eq(SysUser::getStatus,1).ne(SysUser::getAccount,"admin"));
        if(CollectionUtil.isEmpty(list)){
            return null;
        }else {
            List<SysUserDto> resultLst = new ArrayList<>();
            list.forEach(item->{
                SysUserDto sysUserDto = new SysUserDto();
                BeanUtil.copyProperties(item,sysUserDto);
                resultLst.add(sysUserDto);
            });
            return resultLst;
        }
    }

    @Override
    public void delete(List<SysUserParam> sysUserParams) {
        sysUserParams.forEach(item->{
            SysUser sysUser = this.getById(item.getId());
            sysUser.setStatus(0);
            this.updateById(sysUser);
        });
    }

    @Override
    public SysUser getUserByCount(String account) {
        List<SysUser> sysUsers = sysUserMapper.selectList(new QueryWrapper<SysUser>().lambda().eq(SysUser::getAccount, account));
        if(CollectionUtil.isEmpty(sysUsers)){
            return null;
        }else {
            return sysUsers.get(0);
        }
    }

    @Override
    public void modifyPassword(ModifyPasswordParam param) {
        SysUser sysUser = this.getById(param.getId());
        if(org.springframework.security.crypto.bcrypt.BCrypt.checkpw(param.getOldPassword(),sysUser.getPassword())){
            sysUser.setPassword(BCrypt.hashpw(param.getNewPassword(), BCrypt.gensalt()));
            this.updateById(sysUser);
        }else {
            throw new AuthException(AuthExceptionEnum.OLD_PWD_ERROR);
        }
    }

    @Override
    public Boolean checkUserAccount(String account) {
        List<SysUser> sysUsers = sysUserMapper.selectList(new QueryWrapper<SysUser>().lambda().eq(SysUser::getAccount, account));
        return CollectionUtil.isEmpty(sysUsers);
    }
}
